//---------------------------------------------------------------------------
// File: TMainForm.cpp
//
//  TDx_Input_Library
//  Example1 - Enumerating Devices
//
// Date: 1st February, 2002
// Author: Darren John Dwyer
// Copyright: (c) 2002 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to create a DirectInput interface,
// then how to use it to enumerate available DirectInput devices,
// create a selected device, and retrieve information about the device.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3+ only
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Input"
#pragma link "TDx_InputDevice"
#pragma link "TDIDevCaps"
#pragma link "TDx_Input_Library_Install"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Width=780;
Height=580;
Position=poScreenCenter;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Input1->Created) Dx_Input1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateInterfaceButtonClick(TObject *Sender)
{
Msg("Creating DirectInput Interface");

if (Dx_Input1->Created)
    Dx_Input1->Destroy();

Dx_Input1->Create();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnumerateButtonClick(TObject *Sender)
{
Msg("Scanning for Available Devices");

NumDevices = 0;
Dx_Input1->EnumDevices(0, NULL, DIEDFL_ALLDEVICES );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateSelectedButtonClick(TObject *Sender)
{
Msg("Creating Selected Device");

if (Dx_InputDevice1->Created)
    Dx_InputDevice1->Destroy();

int index = DevicesListBox->ItemIndex;
if (DevicesListBox->Items->Strings[index] == "Keyboard") Dx_InputDevice1->Create( GUID_SysKeyboard, Dx_Input1 );
else if (DevicesListBox->Items->Strings[index] == "Mouse") Dx_InputDevice1->Create( GUID_SysMouse, Dx_Input1 );
else Dx_Input1->CreateDevice( InstanceGuids[index], Dx_InputDevice1 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RetrieveCapsButtonClick(TObject *Sender)
{
Msg("Retrieving Device Capabilities");
DeviceCapsListBox->Items->Clear();
if (Dx_InputDevice1->GetCapabilities( DIDevCaps1 ))
    {
    DeviceCapsListBox->Items->Add( "Axis: "+IntToStr( (int) DIDevCaps1->Axes) );
    DeviceCapsListBox->Items->Add( "Buttons: "+IntToStr( (int) DIDevCaps1->Buttons) );
    DeviceCapsListBox->Items->Add( "DevType: "+IntToStr( (int) DIDevCaps1->DevType) );
    DeviceCapsListBox->Items->Add( "FFDriverVersion: "+IntToStr( (int) DIDevCaps1->FFDriverVersion) );
    DeviceCapsListBox->Items->Add( "FFMinTimeResolution: "+IntToStr( (int) DIDevCaps1->FFMinTimeResolution) );
    DeviceCapsListBox->Items->Add( "FFSamplePeriod: "+IntToStr( (int) DIDevCaps1->FFSamplePeriod) );
    DeviceCapsListBox->Items->Add( "FirmwareRevision: "+IntToStr( (int) DIDevCaps1->FirmwareRevision) );
    DeviceCapsListBox->Items->Add( "Flags:" );
    DeviceCapsListBox->Items->AddStrings( DIDevCaps1->Flags_Strings );
    DeviceCapsListBox->Items->Add( "HardwareRevision: "+IntToStr( (int) DIDevCaps1->HardwareRevision) );
    DeviceCapsListBox->Items->Add( "POV's: "+IntToStr( (int) DIDevCaps1->POVs) );
    }
else
    {
    DeviceCapsListBox->Items->Add( "------ Unknown ------" );
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyActiveButtonClick(TObject *Sender)
{
Msg("Destroying Active Device");
if (Dx_InputDevice1->Created) Dx_InputDevice1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyInterfaceButtonClick(TObject *Sender)
{
if (Dx_Input1->Created) Dx_Input1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearMessagesButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Create(TObject *Sender)
{
Msg("DirectInput Interface has been created");
CreateInterfaceButton->Enabled = false;
EnumerateButton->Enabled = true;
CreateSelectedButton->Enabled = false;
DestroyInterfaceButton->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Destroy(TObject *Sender)
{
Msg("DirectInput interface is about to be destroyed." );
if (Dx_InputDevice1->Created) Dx_InputDevice1->Destroy();
DestroyInterfaceButton->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("ERROR ["+Error+"]: "+Function+", "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1EnumDevices(TObject *Sender,
    TDIDeviceInstance *DeviceInstance, void *Context, bool &Finished)
{
if (NumDevices<MAX_DEVICES)
    {
    DevTypes[NumDevices] = DeviceInstance->DevType;
    FFDriverGuids[NumDevices] = DeviceInstance->FFDriver;
    InstanceGuids[NumDevices] = DeviceInstance->Instance;
    InstanceNames[NumDevices] = DeviceInstance->InstanceName;
    ProductGuids[NumDevices] = DeviceInstance->Product;
    ProductNames[NumDevices] = DeviceInstance->ProductName;
    Usages[NumDevices] = DeviceInstance->Usage;
    UsagePages[NumDevices] = DeviceInstance->UsagePage;
    NumDevices++;

    Msg("Found a device: "+DeviceInstance->ProductName);
    DevicesListBox->Items->Add( DeviceInstance->ProductName );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DevicesListBoxClick(TObject *Sender)
{
if (DevicesListBox->Items->Count>0)
    {
    int index = DevicesListBox->ItemIndex;
    AnsiString device_name = DevicesListBox->Items->Strings[index];
    Msg("Device Selected: '"+device_name+"'");

    CreateSelectedButton->Enabled = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1Create(TObject *Sender)
{
Msg("Input Device has been Created");
RetrieveCapsButton->Enabled = true;
DestroyActiveButton->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1Destroy(TObject *Sender)
{
Msg("Input Device is about to be Destroyed");
CreateSelectedButton->Enabled = true;
RetrieveCapsButton->Enabled = false;
DestroyActiveButton->Enabled = false;
}
//---------------------------------------------------------------------------
