//---------------------------------------------------------------------------
// File: TMainForm.h, TDx_Input_Library, Example2 - Keyboard
//
// Date: 1st February, 2002
// Author: Darren John Dwyer
// Copyright: (c) 2002 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to talk to the keyboard using DirectInput.
// It shows two/three (2/3) techniques of accessing the keyboard data :-
//
// (1) Polled Mode
// (2) Event-Driven Mode
// (3) Event-Drive + Forced Polled Mode
//
// -----------
// Polled Mode
// -----------
//
// To access the data from a DirectInput device, you need to ask it
// questions regarding it's current status.
// For example: is a particular key currently pressed on the keyboard?
//
// To this end, DirectInput requires you to force the update of the device
// status, periodically, using the TDx_InputDevice::Poll() method, to make
// certain that the device data retrieved using TDx_InputDevice::GetDeviceState()
// is accurate as possible.
//
// Hence, I've setup a TTimer() to poll the device and retrieve information.
//
// -----------------
// Event-Driven Mode
// -----------------
//
// Some devices can talk with the rest of the computer better than others.
// These devices are smart enough to generate standard win32-events into
// the windows event queue.
//
// The TDx_Library actively monitors and redirects these difficult to use
// events into easy to use TDx_InputDevice::OnDeviceStateChange() events,
// when enabled by calling TDx_InputDevice::SetEventNotifications(true).
//
// -------------------------------
// Event-Driven + Forced Poll Mode
// -------------------------------
//
// Overall, the event-driven method works best, and I suggest you use it first.
//
// Unfortunately, not all input devices (my keyboard for example) do not
// generate the required win32 events, and hence some forced polling is required
// in order to accomodate these events.
//
// This example has been created to handle Event-Driven if possible, and use
// forced polling when not possible.
//
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include "TDx_Input.h"
#include "TDx_Input_Library_Install.h"
#include <vcl\Buttons.hpp>
#include "TDx_InputDevice.h"
#include "TDIDevCaps.h"
#include "TDIKeyboardState.h"
#include <vcl\ExtCtrls.hpp>
//---------------------------------------------------------------------------

#if (__BORLANDC__ < 0x0530) // BCB Version 1
class TMainForm : public TForm
{
#else // BCB Version 3 or 4
class PACKAGE TMainForm : public TForm
{
#endif

__published:
//---------------------------------------------------------------------------
// Anything that is to be displayed within the Object Inspector
//---------------------------------------------------------------------------

    TDx_Input *Dx_Input1;
    TDx_InputDevice *Dx_InputDevice1;
    TDIDevCaps *DIDevCaps1;
    TDIKeyboardState *DIKeyboardState1;
    TTimer *PollingTimer;
    TTimer *ForcedPollTimer;
    TPanel *Panel5;
    TLabel *Label2;
    TButton *CreateButton;
    TBitBtn *ExitButton;
    TButton *BeginPoll;
    TButton *EndPoll;
    TButton *BeginThread;
    TButton *EndThread;
    TButton *DestroyButton;
    TPanel *Panel1;
    TLabel *Label3;
    TMemo *Memo1;
    TPanel *Panel2;
    TLabel *Label1;
    TLabel *KeysPressed;
    TPanel *Panel3;
    TLabel *Label4;
    TMemo *StatusMemo;
    TBitBtn *ClearMessagesButton;
    void __fastcall FormCreate(TObject *Sender);
    void __fastcall ExitButtonClick(TObject *Sender);
    void __fastcall ClearMessagesButtonClick(TObject *Sender);
    void __fastcall Dx_Input1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage);
    void __fastcall Dx_Input1Create(TObject *Sender);
    void __fastcall Dx_Input1Destroy(TObject *Sender);
    void __fastcall Dx_InputDevice1Create(TObject *Sender);
    void __fastcall Dx_InputDevice1Destroy(TObject *Sender);
    void __fastcall BeginPollClick(TObject *Sender);
    void __fastcall PollingTimerTimer(TObject *Sender);
    void __fastcall EndPollClick(TObject *Sender);
    void __fastcall Dx_InputDevice1DeviceStateChange(TObject *Sender);
    void __fastcall BeginThreadClick(TObject *Sender);
    void __fastcall EndThreadClick(TObject *Sender);
    void __fastcall ForcedPollTimerTimer(TObject *Sender);
    void __fastcall FormHide(TObject *Sender);
    void __fastcall CreateButtonClick(TObject *Sender);
    void __fastcall DestroyButtonClick(TObject *Sender);
public:
//---------------------------------------------------------------------------
// Anything available to other classes and/or code
//---------------------------------------------------------------------------

    __fastcall TMainForm(TComponent* Owner);
    virtual __fastcall ~TMainForm();

protected:
//---------------------------------------------------------------------------
// Anything available only to classes descended from this class
//---------------------------------------------------------------------------

private:
//---------------------------------------------------------------------------
// Private data not accessible except from within this class
//---------------------------------------------------------------------------

    #define MAX_DEVICES 5

    dword       DevTypes[MAX_DEVICES];
    GUID        FFDriverGuids[MAX_DEVICES];
    GUID        InstanceGuids[MAX_DEVICES];
    AnsiString  InstanceNames[MAX_DEVICES];
    GUID        ProductGuids[MAX_DEVICES];
    AnsiString  ProductNames[MAX_DEVICES];
    word        Usages[MAX_DEVICES];
    word        UsagePages[MAX_DEVICES];
    int         NumDevices;

    bool        Polled_Mode;
    bool        Forced_Polling_Required;
    bool        Device_Has_Been_Acquired;

};
//---------------------------------------------------------------------------
#if (__BORLANDC__ < 0x0530) // BCB Version 1
extern TMainForm *MainForm;
#else // BCB Version 3 +
extern PACKAGE TMainForm *MainForm;
#endif
//---------------------------------------------------------------------------
#endif
